#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <EEPROM.h>
#include "Index.h"
#include "WEBset.h"
extern "C" {
#include "user_interface.h"
}

#define DNS
#define Taster1 2
#define Taster2 0
#define Taster3 4
#define Taster4 5
#define Relay1 13
#define Relay2 12
#define Relay3 14
#define Relay4 16
int active_low = 1;    // "0" beim schalten mit +5V , "1" beim schalten mit 0V

IPAddress apIP(192, 168, 168, 30);

#if defined DNS
#include <DNSServer.h>
const byte DNS_PORT = 53;
DNSServer dnsServer;
#endif

String BenuzPass = "";
char ssid[32] = "fhgdhfghd\0";
char passwort[64] = "gghdfgh\0";
char BenuzName[32] = "admin\0";
char BenuzPasswort[32] = "gghdfgh\0";

ESP8266WebServer server;
EspClass esp;
int z = 0;                   //Aktuelle EEPROM-Adresse zum lesen
#include "Setup.h"

int i = 0;
int val1, val2 , val3, val4;
String Temp = "";
boolean AP = 0, WLAN_Fehlt = 0;; // Acsespoint Modus aus

void setup()                // Wird 1 Mal beim Start ausgefuehrt
{
  char inser;               // Serielle daten ablegen
  String nachricht = "";    //  Setup Formular
  pinMode(Relay1, OUTPUT);
  pinMode(Relay2, OUTPUT);
  pinMode(Relay3, OUTPUT);
  pinMode(Relay4, OUTPUT);
  digitalWrite(Relay1, active_low);
  digitalWrite(Relay2, active_low);
  digitalWrite(Relay3, active_low);
  digitalWrite(Relay4, active_low);
  pinMode(Taster1, INPUT_PULLUP);
  pinMode(Taster2, INPUT_PULLUP);
  pinMode(Taster3, INPUT_PULLUP);
  pinMode(Taster4, INPUT_PULLUP);


  EEPROM.begin(250);                                 // EEPROM initialisieren mit 200 Byts
  Serial.begin(115200);

  while (Serial.available())inser = Serial.read();  // Setup Weiche, Erkennen ob TXD/RXD gebrückt sind
  Serial.println("Start");
  delay(10);
  inser = Serial.read();
  if (inser != 'S') goto weiter;
  inser = Serial.read();
  if (inser != 't') goto weiter;
  Einstellen();

weiter:
  z = 0;
  LeseEeprom(ssid, sizeof(ssid));        // EEPROM lesen
  if (ssid[0] == 255) // Wenn EEPROM leer ist dann:
  {
    for (i = 0; i < 6; i++) EEPROM.write(i, '\0');
    EEPROM.commit();
    Einstellen();
  }

  LeseEeprom(passwort, sizeof(passwort));
  LeseEeprom(BenuzPasswort, sizeof(BenuzPasswort));

  if (ssid[0] == '\0')
  {
    Serial.println("Starte in Access Point modus" );
    Serial.println("IP http://192.168.168.30");
    Serial.print("SSID: WebSchalter, Passwort: ");
    Serial.println(passwort);
    WiFi.mode(WIFI_AP);      // access point modus
    WiFi.softAPConfig(apIP, apIP, IPAddress(255, 255, 255, 0));
    strncpy(ssid, "WebSchalter\0", 13);
    WiFi.softAP(&ssid[0], &passwort[0]);  // Name des Wi-Fi netzes
#if defined DNS
    dnsServer.start(DNS_PORT, "*", apIP);
#endif
    AP = 1;
  }
  else // Wenn ssid angegeben dann in Stationmodus mit Router verbinden
  {
    Serial.print("Verbinde mit ");
    Serial.println(ssid);
    WiFi.mode(WIFI_STA);
    WiFi.begin(ssid, passwort);
    int timout = 0;
    while (WiFi.status() != WL_CONNECTED)
    {
      delay(500);
      Serial.print("O");
      timout++;
      if  (timout > 20) // Wenn Anmeldung nicht möglich
      {
        Serial.println("");
        Serial.println("Wlan verbindung fehlt");
        WLAN_Fehlt = 1;
        break;
        //return;
      }
    }
    if (WiFi.status() == WL_CONNECTED)
    {
    Serial.println("");
    Serial.println("Mit Wlan verbunden");
    Serial.println("IP Adresse: ");
    Serial.println(WiFi.localIP());
    }
  }// ENDE Stationmodus / Access Point modus Auswahl

  if (!BenuzPasswort[0])Serial.println("Serverauthentifizierung ausgeschaltet");
  else
  {
    Serial.println("Serverauthentifizierung eingeschaltet");
    Serial.print("Benutzername : ");
    Serial.println(BenuzName);
    Serial.print("Benutzerpasswort : ");
    Serial.println(BenuzPasswort);
  }

  //  Bechandlung der Ereignissen anschlissen
  server.on("/", Ereignis_Index);
  server.on("/schalte.php", Ereignis_Schalte);
  server.on("/zustand.php", Ereignis_Zuschtand);
  server.onNotFound ( handleNotFound );

  server.begin();  // Starte den Server
  Serial.println("HTTP Server gestartet");
}

void Ereignis_Schalte()
{
  if (BenuzPasswort[0]) // Wenn Benutzerpasswort angegeben wurde
  {
    if (!server.authenticate(&BenuzName[0], &BenuzPasswort[0]))
      return server.requestAuthentication();
  }
  String RelayNr = server.arg("Relay");          // Relais Nr. empfangen
  int RelayNrInt = RelayNr.toInt();
  // String zu Integer konvertieren
  String ArgumentOn = server.arg("On");
  int RelayOn = ArgumentOn.toInt();

  switch (RelayNrInt)
  {
    case 1:
      if (RelayOn == 1) val1 = 1;
      if (RelayOn == 0) val1 = 0;
      break;
    case 2:
      if (RelayOn == 1) val2 = 1;
      if (RelayOn == 0) val2 = 0;
      break;
    case 3:
      if (RelayOn == 1) val3 = 1;
      if (RelayOn == 0) val3 = 0;
      break;
    case 4:
      if (RelayOn == 1) val4 = 1;
      if (RelayOn == 0) val4 = 0;
      break;
  }
  digitalWrite(Relay1, active_low ^ val1);
  digitalWrite(Relay2, active_low ^ val2);
  digitalWrite(Relay3, active_low ^ val3);
  digitalWrite(Relay4, active_low ^ val4);
  Ereignis_Zuschtand();
}

void Ereignis_Zuschtand()
{
  if (BenuzPasswort[0]) // Wenn Benutzerpasswort angegeben wurde
  {
    if (!server.authenticate(&BenuzName[0], &BenuzPasswort[0]))
      return server.requestAuthentication();
  }
  String Antwort = String(val1) + ";" + String(val2) + ";" + String(val3) + ";" + String(val4) + ";";
  server.sendHeader("Cache-Control", "no-cache");
  server.send(200, "text/plain", Antwort); // Antwort an Internet Browser senden
}

void Ereignis_Index()    // Wird ausgeuehrt wenn "http://<ip address>/" aufgerufen wurde
{
  if (BenuzPasswort[0]) // Wenn Benutzerpasswort angegeben wurde
  {
    if (!server.authenticate(&BenuzName[0], &BenuzPasswort[0]))
      return server.requestAuthentication();
  }
  server.sendHeader("Cache-Control", "no-cache");
  server.send(200, "text/html", indexHtml);
}

/*
  void Ereignis_Index()    // Wird ausgeuehrt wenn "http://<ip address>/" aufgerufen wurde
  {
  if(!server.authenticate(&ssid[0], &passwort[0]))
      return server.requestAuthentication();
    server.send(200, "text/plain", "Login OK");
  }

*/
void loop()
{
#if defined DNS
  if (AP)dnsServer.processNextRequest();
#endif

  if (!digitalRead(Taster1)) //Taster betätigt
  {
    delay(50);
    if (!digitalRead(Taster1)) // Nach 100mS immer noch betätigt dann ist es keine Induktionstrom in der Leitung
    {
      val1 = !val1;
      digitalWrite(Relay1, active_low ^ val1);
      while (!digitalRead(Taster1))server.handleClient(); // Warten bis der Taster losgelassen wird
    }
  }

  if (!digitalRead(Taster2)) //Taster betätigt
  {
    delay(50);
    if (!digitalRead(Taster2)) // Nach 100mS immer noch betätigt dann ist es keine Induktionstrom in der Leitung
    {
      val2 = !val2;
      digitalWrite(Relay2, active_low ^ val2);
      while (!digitalRead(Taster2))server.handleClient(); // Warten bis der Taster losgelassen wird
    }
  }

  if (!digitalRead(Taster3)) //Taster betätigt
  {
    delay(50);
    if (!digitalRead(Taster3)) // Nach 100mS immer noch betätigt dann ist es keine Induktionstrom in der Leitung
    {
      val3 = !val3;
      digitalWrite(Relay3, active_low ^ val3);
      while (!digitalRead(Taster3))server.handleClient(); // Warten bis der Taster losgelassen wird
    }
  }

  if (!digitalRead(Taster4)) //Taster betätigt
  {
    delay(50);
    if (!digitalRead(Taster4)) // Nach 100mS immer noch betätigt dann ist es keine Induktionstrom in der Leitung
    {
      val4 = !val4;
      digitalWrite(Relay4, active_low ^ val4);
      while (!digitalRead(Taster4))server.handleClient(); // Warten bis der Taster losgelassen wird
    }
  }

  server.handleClient();                           // Server Ereignisse abarbeiten
  if (WLAN_Fehlt)
  {
    if (WiFi.status() == WL_CONNECTED)
    {
      Serial.print("Mit Wlan nachtraeglich verbunden -> ");
      Serial.print("IP Adresse: ");
      Serial.println(WiFi.localIP());
      WLAN_Fehlt = 0;
    }
  } 
  else
  {
    if (WiFi.status() != WL_CONNECTED && !AP)
    {
      Serial.println("WLAN verbindung abgebrochen");
      WLAN_Fehlt = 1;
    }
  }

}




